@@ -18,7 +18,7 @@ from oslo_utils import versionutils
 
 from nova import availability_zones
 from nova import context as nova_context
-from nova import db
+from nova.db import api as db
 from nova import exception
 from nova.notifications.objects import base as notification
 from nova.notifications.objects import service as service_notification
@@ -31,7 +31,7 @@ LOG = logging.getLogger(__name__)
 
 
 # NOTE(danms): This is the global service version counter
-SERVICE_VERSION = 30
+SERVICE_VERSION = 35
 
 
 # NOTE(danms): This is our SERVICE_VERSION history. The idea is that any
@@ -131,6 +131,21 @@ SERVICE_VERSION_HISTORY = (
     {'compute_rpc': '4.22'},
     # Version 30: Compute RPC version 5.0
     {'compute_rpc': '5.0'},
+    # Version 31: The compute manager checks if 'trusted_certs' are supported
+    {'compute_rpc': '5.0'},
+    # Version 32: Add 'file_backed_memory' support. The service version bump is
+    # needed to allow the destination of a live migration to reject the
+    # migration if 'file_backed_memory' is enabled and the source does not
+    # support 'file_backed_memory'
+    {'compute_rpc': '5.0'},
+    # Version 33: Add support for check on the server group with
+    # 'max_server_per_host' rules
+    {'compute_rpc': '5.0'},
+    # Version 34: Adds support to abort queued/preparing live migrations.
+    {'compute_rpc': '5.0'},
+    # Version 35: Indicates that nova-compute supports live migration with
+    # ports bound early on the destination host using VIFMigrateData.
+    {'compute_rpc': '5.0'},
 )
 
 
