@@ -123,14 +123,15 @@ class ComputeDriver(object):
 
     capabilities = {
         "has_imagecache": False,
-        "supports_recreate": False,
+        "supports_evacuate": False,
         "supports_migrate_to_same_host": False,
         "supports_attach_interface": False,
         "supports_device_tagging": False,
         "supports_tagged_attach_interface": False,
         "supports_tagged_attach_volume": False,
         "supports_extend_volume": False,
-        "supports_multiattach": False
+        "supports_multiattach": False,
+        "supports_trusted_certs": False,
     }
 
     requires_allocation_refresh = False
@@ -157,10 +158,15 @@ class ComputeDriver(object):
         """
         pass
 
-    def get_info(self, instance):
+    def get_info(self, instance, use_cache=True):
         """Get the current status of an instance.
 
         :param instance: nova.objects.instance.Instance object
+        :param use_cache: boolean to indicate if the driver should be allowed
+                          to use cached data to return instance status.
+                          This only applies to drivers which cache instance
+                          state information. For drivers that do not use a
+                          cache, this parameter can be ignored.
         :returns: An InstanceInfo object
         """
         # TODO(Vek): Need to pass context in for access to auth_token
@@ -231,7 +237,7 @@ class ComputeDriver(object):
     def rebuild(self, context, instance, image_meta, injected_files,
                 admin_password, allocations, bdms, detach_block_devices,
                 attach_block_devices, network_info=None,
-                recreate=False, block_device_info=None,
+                evacuate=False, block_device_info=None,
                 preserve_ephemeral=False):
         """Destroy and re-make this instance.
 
@@ -263,7 +269,7 @@ class ComputeDriver(object):
             nova.compute.manager.ComputeManager:_rebuild_default_impl for
             usage.
         :param network_info: instance network information
-        :param recreate: True if the instance is being recreated on a new
+        :param evacuate: True if the instance is being recreated on a new
             hypervisor - all the cleanup of old state is skipped.
         :param block_device_info: Information about block devices to be
                                   attached to the instance.
@@ -272,6 +278,34 @@ class ComputeDriver(object):
         """
         raise NotImplementedError()
 
+    def prepare_for_spawn(self, instance):
+        """Prepare to spawn instance.
+
+        Perform any pre-flight checks, tagging, etc. that the virt driver
+        must perform before executing the spawn process for a new instance.
+
+        :param instance: nova.objects.instance.Instance
+                         This function should use the data there to guide
+                         the creation of the new instance.
+        """
+        pass
+
+    def failed_spawn_cleanup(self, instance):
+        """Cleanup from the instance spawn.
+
+        Perform any hypervisor clean-up required should the spawn operation
+        fail, such as the removal of tags that were added during the
+        prepare_for_spawn method.
+
+        This method should be idempotent.
+
+        :param instance: nova.objects.instance.Instance
+                         This function should use the data there to guide
+                         the creation of the new instance.
+
+        """
+        pass
+
     def spawn(self, context, instance, image_meta, injected_files,
               admin_password, allocations, network_info=None,
               block_device_info=None):
@@ -830,6 +864,11 @@ class ComputeDriver(object):
         """
         raise NotImplementedError()
 
+    @staticmethod
+    def _get_reserved_host_disk_gb_from_config():
+        import nova.compute.utils as compute_utils  # avoid circular import
+        return compute_utils.convert_mb_to_ceil_gb(CONF.reserved_host_disk_mb)
+
     def update_provider_tree(self, provider_tree, nodename):
         """Update a ProviderTree object with current resource provider and
         inventory information.
@@ -842,21 +881,43 @@ class ComputeDriver(object):
         This method supersedes get_inventory(): if this method is implemented,
         get_inventory() is not used.
 
+        Implementors of this interface are expected to set ``allocation_ratio``
+        and ``reserved`` values for inventory records, which may be based on
+        configuration options, e.g. ``[DEFAULT]/cpu_allocation_ratio``,
+        depending on the driver and resource class. If not provided, allocation
+        ratio defaults to 1.0 and reserved defaults to 0 in placement.
+
         :note: Renaming a provider (by deleting it from provider_tree and
         re-adding it with a different name) is not supported at this time.
 
+        See the developer reference documentation for more details:
+
+        https://docs.openstack.org/nova/latest/reference/update-provider-tree.html   # noqa
+
         :param nova.compute.provider_tree.ProviderTree provider_tree:
-            A ProviderTree object representing all the providers associated
-            with the compute node's nested tree or aggregates.  This is fully
-            owned by the update_provider_tree method, and can therefore be
-            modified without locking/concurrency considerations.
+            A nova.compute.provider_tree.ProviderTree object representing all
+            the providers in the tree associated with the compute node, and any
+            sharing providers (those with the ``MISC_SHARES_VIA_AGGREGATE``
+            trait) associated via aggregate with any of those providers (but
+            not *their* tree- or aggregate-associated providers), as currently
+            known by placement. This object is fully owned by the
+            update_provider_tree method, and can therefore be modified without
+            locking/concurrency considerations. In other words, the parameter
+            is passed *by reference* with the expectation that the virt driver
+            will modify the object. Note, however, that it may contain
+            providers not directly owned/controlled by the compute host. Care
+            must be taken not to remove or modify such providers inadvertently.
+            In addition, providers may be associated with traits and/or
+            aggregates maintained by outside agents. The
+            `update_provider_tree`` method must therefore also be careful only
+            to add/remove traits/aggregates it explicitly controls.
         :param nodename:
-            Name of the compute node for which the caller is updating providers
-            and inventory.  Drivers managing more than one node may use this in
-            an advisory capacity to restrict changes to only the providers
-            associated with that one node, but this is not a requirement: the
-            caller always subsumes all changes regardless.
-        :return: True if the provider_tree was changed; False otherwise.
+            String name of the compute node (i.e.
+            ComputeNode.hypervisor_hostname) for which the caller is requesting
+            updated provider information. Drivers may use this to help identify
+            the compute node provider in the ProviderTree. Drivers managing
+            more than one node (e.g. ironic) may also use it as a cue to
+            indicate which node is being processed by the caller.
         """
         raise NotImplementedError()
 
@@ -866,16 +927,6 @@ class ComputeDriver(object):
         """
         raise NotImplementedError()
 
-    def get_traits(self, nodename):
-        """Get the traits for a given node.
-
-        Returns an iterable of traits for the supplied node.
-
-        :param nodename: the name of the node.
-        :returns: an iterable of traits for the supplied node.
-        """
-        raise NotImplementedError()
-
     def get_available_resource(self, nodename):
         """Retrieve resource information.
 
@@ -1123,10 +1174,6 @@ class ComputeDriver(object):
         - setup_basic_filtering (for nova-basic, etc.)
         - prepare_instance_filter(for nova-instance-instance-xxx, etc.)
 
-        to_xml may have to be called since it defines PROJNET, PROJMASK.
-        but libvirt migrates those value through migrateToURI(),
-        so , no need to be called.
-
         Don't use thread for this method since migration should
         not be started when setting-up filtering rules operations
         are not completed.
